;        TITLE   'Serial Interface Routines
;        PROCESSOR     42

;This is a short program to demonstrate how to transmit and receive
;serial data using the PIC17C42.
;
;A message will be transmitted and routed right back to the processor
;and read. The read information will be saved in an internal buffer.
;
;       Program:          SERINT_A.ASM 
;       Revision Date:   
;                         12-9-97     Compatibility with MPASM, MPLAB V3.22
;
;
        LIST     P = 17C42

#include    <p17c42.inc>

TX_BUFFER       equ     0x80
RX_BUFFER       equ     0xB0
RXPTR           equ     0x20
TXPTR           equ     0x21
SERFLAG         equ     0x22
RTINUM          equ     0x23
RXDONE          equ     1
TXDONE          equ     0
HILOB           equ     2
;
;
        ORG     0
        goto    start
;        
        ORG     0x0010          ;vector for rtcc interrupt
;rtcc_int                       ;not used here
;
        ORG     0x0020          ;vector for peripheral interrupt
perf_int
        goto    service_perf    ;service the interrupts
;
        ORG     0x0030
;
;initialize the serial port: baud rate interrupts etc.
init_serial
        clrf    SERFLAG, F      ;clear all flags
        movlb   0               ;select bank 0
        movlw   0x07            ;select 9600 baud
        movfp   WREG,SPBRG         ;       /
        movlw   0x90            ;set up serial pins
        movfp   WREG,RCSTA         ;       /
        clrf    TXSTA, F        ;setup transmit status
        movlb   1               ;select bank 1
        clrf    PIR, F          ;clear all interrupts
        clrf    PIE, F          ;clear all enables
        bsf     PIE,RCIE        ;enable receive interrupt
        movlw   RX_BUFFER       ;set pointer to rx buffer
        movpf   WREG,RXPTR         ;       /
        clrf    INTSTA, F       ;clear all interrupts
        bsf     INTSTA,PEIE     ;enable peripheral ints
        retfie
;
;start transmission of first two bytes
start_xmit
        movlb   0               ;select bank 0
        bsf     TXSTA,TXEN      ;enable transmit
        tablrd  1,1,WREG           ;load latch
        tlrd    1,TXREG         ;load high byte
        movlb   1               ;select bank 1
empty_chk
        btfss   PIR,TXIF        ;TXBUF empty?
        goto    empty_chk       ;no then keep checking
        movlb   0               ;select bank 0
        tablrd  0,1,TXREG       ;load lo byte
        movlb   1               ;select bank 1
        bsf     PIE,TXIE        ;enable transmit interrupts
        bsf     SERFLAG,HILOB   ;set up next for high byte
        return
;
;
        PAGE
;
service_perf
;check for transmit or receive interrupts only
	movlb   1		;add this line of code here
        btfsc   PIR,RCIF        ;RX buffer full?
        goto    service_recv    ;yes then service
        btfss   PIR,TXIF        ;TX buffer empty?
        goto    exit_perf       ;no, ignore other int.
service_xmt
        btfsc   SERFLAG,TXDONE  ;all done?
        goto    exit_perf       ;yes then quit
        btfsc   SERFLAG,HILOB   ;if clr, do low byte
        goto    rd_hi           ;else read high byte
        tablrd  0,1,WREG           ;read lo
        goto    sx_cont         ;continue
rd_hi
        tlrd    1,WREG             ;read high byte
sx_cont
        btg     SERFLAG,HILOB   ;toggle flag
        movlb   0               ;bsr=0
        movpf   WREG,TXREG         ;load tx reg
        tstfsz  WREG               ;last byte?
        goto    exit_perf       ;no then cont
end_xmt                         ;else end transmit
        movlb   1               ;select bank 1
        bcf     PIE,TXIE        ;disable tx interrupt
        bsf     SERFLAG,TXDONE  ;set done flag
exit_perf
        bcf     INTSTA,PEIF     ;clear peripheral int
        retfie
;
service_recv
        btfsc   SERFLAG,RXDONE  ;RX complete?
        goto    exit_perf       ;exit int
        movfp   RXPTR,FSR0      ;get pointer
        movlb   0               ;select bank 0        
        movfp   RCREG,INDF0     ;load received value
        clrf    WREG, F         ;clr W
        cpfsgt  INDF0           ;value = 0?
        goto    end_recv        ;yes then end
        incf    FSR0, F         ;inc pointer
        movpf   FSR0,RXPTR      ;save pointer
        goto    exit_perf       ;return from int
end_recv
        bsf     SERFLAG,RXDONE  ;set flag
        clrf    INTSTA, F       ;clear all int
        movlb   1               ;select bank 1
        bcf     PIE,RCIE        ;disable rx interrupts
        goto    exit_perf       ;return
        PAGE

;
start
        clrf    FSR1, F         ;assign FSR1 as S.P.
        decf    FSR1, F         ;       /
        movlw   0x20            ;clear ram space
        movfp   WREG,FSR0          ;do indirect addressing
start1
        clrf    INDF0, F        ;clear ram
        incfsz  FSR0, F         ;inc and skip if done
        goto    start1
        call    init_serial     ;initialize serial port
        movlw   LOW   MESSAGE   ;load table pointer
        movpf   WREG,TBLPTRL       ;       /
        movlw   HIGH  MESSAGE    ;     /  
        movpf   WREG,TBLPTRH       ;    /
        call    start_xmit      ;start transmission
chk_end        
        btfss   SERFLAG,RXDONE  ;receive all?
        goto    chk_end         ;no then keep checking
;
loop    goto    loop            ;spin wheel
;
        ORG     0x100
MESSAGE
        DATA    "The code is: Tea for the Tillerman"
        DATA    0
;
;
        END
