 title  "lego_rs4 - Simple PICMicro RS-232 Interface"
;
;	Let's Try the interupt scheme
;
;
;  Hardware Notes:
;   PIC16F84 Running at 4 MHz
;   _MCLR is Pulled Up
;   PORTB.3 is the Transmit Output
;   PORTB.0 is the RS-232 Input
;
;
  LIST R=DEC
  INCLUDE "p16f84.inc"

;  Register Usage
 CBLOCK 0x020                   ;  Start Registers at End of the Values
Byte, Count			;  Variables for RS-232
BTest
Dlay				;  Dlay Count
_w, _status
valid
Bytes:7
i
byte_cnt
byte_loc
 ENDC

#define  RX PORTB, 0		; Data Received from RS232 CPU Port
#define  LED1 PORTA, 0		; Test LED1
#define  LED2 PORTA, 1		; Test LED2
#define  TX PORTB, 3		; Data Sent to RS232 CPU Port

 PAGE
 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF

;  Mainline of lego_rs
  org 	0

  goto MainLine

  org    4

Int

  movwf	_w			; Save the Registers 
  movf	STATUS, w
  movwf _status
  				
  call  HalfBitDlay		; 
  btfss RX
  call 	RXLoop
  
  goto	_IntEnd

_IntEnd

  bcf	INTCON, INTF		; re-enable interupts

  movf	_status, w		; restore registers
  movwf STATUS
  swapf _w, f
  swapf _w, w
  retfie
;******************************************************************************
MainLine 

  clrf	Bytes
  clrf  valid
  clrf  Count
  clrf  byte_cnt

  movlw 0xFF			; DEBUG Value
  movwf Byte			; DEBUG
  movlw 0
  movwf valid


  movlw	b'10101010'			
  movwf BTest

  bsf    STATUS, RP0

  movlw  b'10110000'
  movwf  OPTION_REG		; Set RB0/Int for rising edge interrupt

; Set-up pins for correct directions
; Output Pins

  bcf 	 LED1
  bcf    LED2
  bcf	 TX

; Input Pins
	
  bsf	 RX

  bcf	STATUS, RP0

  BSF OPTION_REG, INTEDG   	; interupt on positive
  BCF INTCON, INTF    		; clear interrupt flag
  BSF INTCON, INTE    		; mask for external interrupts
  BSF INTCON, GIE     		; enable interrupts

Loop
  
  movf byte_cnt, w 		; check to see if we have 7 bytes
  xorlw 6
  btfsc STATUS,Z
  call  TXLoop

  movf valid, w			; check to see if we have a valid
  xorlw 1			; serial transmission
  btfsc STATUS,Z

  call  Byte_inc	

  goto	Loop  
;******************************************************************************
RXLoop				; Loop Here to Read in the Byte

  movlw	 8			; Load 8 into Count for 8 bits
  movwf	 Count

RX_inner

  call   BitDlay		;  Wait a Full Byte

  bcf    STATUS, C		;  Set Carry Accordingly
  btfss  RX			;  Bit High or Low?
   bsf   STATUS, C

  rrf    Byte, f		;  Shift in the Byte

  goto   $ + 1			;  Make 11 Cycles in Loop
  goto   $ + 1

  decfsz Count, f
   goto  RX_inner

  call   BitDlay		;  Make Sure there is a "Stop" Bit

  btfsc  RX
   goto  _IntEnd		;  Not High, Then No Byte

  movlw	 1			; Set the Valid Bit Flag
  movwf	 valid

  return

;******************************************************************************
TXLoop
  
  BCF INTCON, GIE     		; disable interrupts

  clrf  byte_cnt		; reset the byte count
				;  (should use this to decrement for TX instead!)
  movlw	10
  movwf Count
  bcf	STATUS, C
  comf  Byte, f			; Complement the received value
				;  (Not Sure why it's inverted however 9/3/02!)
TX_inner

  btfsc  STATUS, C		;  Send the Bit in "Carry"
   goto  $ + 4
  nop				;  Send a "Low"
  bcf    TX
  goto   $ + 3
  bsf    TX			;  Send a "High"
  goto   $ + 1			;  6 Cycles in Loop

  call   BitDlay		;  Wait a Bit

  bsf    STATUS, C		;  Shift Out the Next Bit into Carry
  rrf    Byte, f

  decfsz Count, f		;  11 Intrinsic Delays in TXLoop
   goto  TX_inner

  call   toggle2

  BSF INTCON, GIE     		; disable interrupts

  return
;******************************************************************************
BitDlay				;  Delay 833 - 15 Cycles (including Call/Return)

  movlw  100
  addlw  0x0FF			;  Take 1 Away from the Loop
  btfss  STATUS, Z
   goto  $ - 2

  goto   $ + 1

  return
;******************************************************************************


;******************************************************************************
HalfBitDlay			;  Delay (833 - 15) / 2 Cycles 

  movlw  50
  addlw  0x0FF			;  Take 1 Away from the Loop
  btfss  STATUS, Z
   goto  $ - 2

  return
;******************************************************************************
toggle

  btfsc LED1
  goto togclr
  bsf LED1
  return

togclr

  bcf LED1
  return
;******************************************************************************
toggle2

  btfsc LED2
  goto togclr2
  bsf LED2
  return

togclr2

  bcf LED2
  return
;******************************************************************************
Byte_inc
  
  movlw Bytes			; update array location
  addwf byte_cnt, w  

  movwf byte_loc		; store array index

  incf byte_cnt,f		; increment array index pointer

  movwf FSR			; update FSR with current array location		

  movf  Byte, w			; get the latest serial Byte
  movwf	INDF			; update the current array location 
  
  movf byte_loc, w		; Restore the array pointer with
  subwf FSR, f			; with the start of the array

  clrf  valid			; reset the valid transmission flag

  call  toggle2

  return
;******************************************************************************
 end
