;title  "uart_2 - Simple 3-Wire RS-232 Application"
;
;
;  Hardware Notes:
;   16F628 Running at 16 MHz
;   _MCLR is Pulled Up
;   
;

  LIST P=16F628, R=DEC
  INCLUDE "p16f628.inc"

 PAGE
 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF ^ _BODEN_ON

;Register Usage
  CBLOCK 0x020
Bytes:9
Byte_cnt
Byte_loc
Byte_dec
Byte
Old_B
New_B
  ENDC

#define ClkFreq		16000000		; clock frequency
#define baud(X)		((10*ClkFreq/(64*X))+5)/10-1
#define TXSTA_INIT	0xA0			
#define RCSTA_INIT	b'10010000'
#define LED1		PORTA, 0
#define LED2		PORTA, 1
#define int_pin		PORTB, 0
#define int_TX		PORTB, 6
#define int_RX		PORTB, 7		

  org    0

  goto	 SetUp

  org	 4

Int

RXLoop

  movf	  Byte_cnt, w		; check to see if we have right number of Bytes
  xorlw	  8
  btfsc	  STATUS, Z
   goto   RX_test
  btfss   PIR1, RCIF            ;  Wait for Received Character Set
   goto   RXLoop                ;   - Nothing There
  movf    RCREG, Byte	        ;  Get the Received Character
  bcf     PIR1, RCIF	        ;  Clear the Character Present Flag
  call    Byte_inc
   goto	  RXLoop

TXLoop

  movlw  baud(14400)		;  Make the Serial Port Run at 14400 bps
  movwf  SPBRG

  movwf	 Byte_cnt
  movf   Byte_dec, w
  clrf   Byte_cnt  

TX_inner

  decf   Byte_dec   
  btfsc	 STATUS, Z
   goto   _IntEnd
  call	 Byte_read
  movf   Byte, w
  movwf  TXREG			;  Send the Character
   goto	 TX_inner  

_IntEnd

  movlw  baud(2400)		;  Make the Serial Port Run at 2400 bps
  movwf  SPBRG
 
RX_test
  bsf    PIE1, RCIE
  call   toggle

  retfie
;****************************************************
SetUp 

  bsf    STATUS, RP0		; Goto Bank 0

  bcf	 LED1			; Set LED1 as an Output

  movlw  baud(2400)		;  Make the Serial Port Run at 2400 bps
  movwf  SPBRG
  movlw  TXSTA_INIT
  movwf  TXSTA
  movlw	 RCSTA_INIT
  bcf    STATUS, RP0
  movwf  RCSTA 			;  Enable Serial I/O

  bsf    PIE1, RCIE

  bcf	 LED1

Loop	 
  nop
  goto Loop

;****************************************************
toggle
  btfsc	LED1
  goto  togclr
  bsf   LED1
  return
;****************************************************
togclr
  bcf 	LED1
  return  
;****************************************************
Byte_inc
  
  movlw Bytes			; update array location
  addwf Byte_cnt, w  

  movwf Byte_loc		; store array index

  incf Byte_cnt,f		; increment array index pointer

  movwf FSR			; update FSR with current array location		

  movf  Byte, w			; get the latest serial Byte
  movwf	INDF			; update the current array location 
  
  movf Byte_loc, w		; Restore the array pointer with
  subwf FSR, f			;  the start of the array

  return
;****************************************************
Byte_read
  
  movlw Bytes			; update array location
  addwf Byte_cnt, w  

  movwf Byte_loc		; store array index

  incf Byte_cnt,f		; increment array index pointer

  movwf FSR			; update FSR with current array location		

  movf  INDF, w			; get the latest serial Byte
  movwf	Byte			; update the current array location 
  
  movf Byte_loc, w		; Restore the array pointer with
  subwf FSR, f			;  the start of the array

  return
;****************************************************
 end
